/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.base;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * An openeas servlet request
 */
public class ESServletRequest {

  OpenClient client_ = null;
  HttpServletRequest req_ = null;
  HttpServletResponse rsp_ = null;
  Hashtable params_ = new Hashtable();

  public ESServletRequest(HttpServletRequest req, HttpServletResponse rsp, OpenClient client) {
    req_ = req;
    rsp_ = rsp;
    client_ = client;
  }

  public String getParameter(String name) {
    // First check user-added parameters:
    // the user may override an existing param
    String ret = (String)params_.get(name);
    if(ret == null && req_ != null) {
      ret = req_.getParameter(name);
    }
    return ret;
  }

  public int getIntParam(String name, int errval) {
    try {
      String val = getParameter(name);
      if(val == null) return errval;
      else return Integer.parseInt(val);
    } catch(NumberFormatException e) {
      return errval;
    }
  }

  public double getDoubleParam(String name, double errval) {
    try {
      String val = getParameter(name);
      if(val == null) return errval;
      else return Double.valueOf(val).doubleValue();
    } catch(NumberFormatException e) {
      return errval;
    }
  }

  public long getLongParam(String name, long errval) {
    try {
      String val = getParameter(name);
      if(val == null) return errval;
      else return Long.valueOf(val).longValue();
    } catch(NumberFormatException e) {
      return errval;
    }
  }

  /**
   * Check wether a parameter comes from the web or is a local variable.
   * May be used for security checks...
   * Note that a parameter overridden by a local variable is considered local.
   */
  public boolean isHttpParameter(String name) {
    if(params_.get(name) != null) return false;
    return (req_.getParameter(name) != null);
  }

  /**
   * Add a new parameter, or set the value of an existing one.
   * @param name the parameter name
   * @param val the parameter value
   */
  public void addParameter(String name, String val) {
    if(name != null && val != null) params_.put(name, val); 
  }

  public String[] getParameterValues(String name) {
    String tmp[] = null;
    if(req_ != null)  tmp = req_.getParameterValues(name);

    String p = (String)params_.get(name);
    if(p == null) return tmp;

    int sz = (tmp == null ? 1 : tmp.length + 1);
    String ret[] = new String[sz];
    ret[0] = p;
    if(tmp != null && sz > 1) {
      for(int i=1; i<=tmp.length; i++) ret[i]=tmp[i-1];
    }
    return ret;
  }

  public void addParameterValues(String name, String[] val) {
    if(name != null && val != null) params_.put(name, val); 
  }

  /**
   * Check the presence of a list of parameters
   * @param names The parameter names
   * @param all If true, return the list of all missing parameters; If false,
   * return only the 1st missing parameter name.
   * @return The name of the first undefined parameter, null if they are
   * all defined
   */
  public String checkParameters(String[] names, boolean all) {
    StringBuffer buf = new StringBuffer();
    boolean err = false;
    for(int i=0; i<names.length; i++) {
      String p = getParameter(names[i]);
      if(p == null || p.trim().length() < 1) {
        err = true;
        if(!all) return names[i];
        else buf.append(names[i] + ',');
      }
    }
    if(err) return buf.toString();
    else return null;
  }

  public Enumeration getParameterNames() {
    Vector v = new Vector();
    if(req_ != null) {
      Enumeration en = req_.getParameterNames();
      while(en.hasMoreElements()) {
        v.addElement(en.nextElement().toString());
      }
    }
    Enumeration en = params_.keys();
    while(en.hasMoreElements()) {
      String s = en.nextElement().toString();
      if(! v.contains(s)) v.addElement(s);
    }
    return v.elements();
  }

  public Hashtable getAllParams() {
    Enumeration names = getParameterNames();
    Hashtable h = new Hashtable();
    while(names.hasMoreElements()) {
      StringBuffer b = new StringBuffer();
      String name = (String)names.nextElement();
      String vals[] = getParameterValues(name);
      for(int i=0; i<vals.length; i++) {
        if(i >= vals.length-1) b.append(vals[i]);
        else b.append(vals[i] + ",");
      }
      h.put(name, b.toString());
    }
    return h;
  }

  public String toString() { return getAllParams().toString(); }

  public HttpServletRequest getServletRequest() { return req_; }
  public HttpServletResponse getServletResponse() { return rsp_; }
};

