/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.db;

import org.experlog.openeas.api.DataSet;
import org.experlog.openeas.api.DataObject;
import org.experlog.gencode.DynHtml;
import org.experlog.gencode.ESDynHtml;
import org.experlog.base.OpenClient;
import org.experlog.base.ESServletRequest;

import java.io.File;
import java.io.PrintWriter;
import java.io.IOException;
import java.sql.SQLException;

public class ESDbFormHandler {

    OpenClient session_;
    ESServletRequest request_ = null;

    DynHtml expander_;

    public ESDbFormHandler(OpenClient s) {
        session_ = s;
        expander_ = new ESDynHtml(s, null);
    }

    public ESDbFormHandler(OpenClient s, ESServletRequest req) {
        this(s);
        request_ = req;
        expander_.setServletRequest(request_);
    }

    /**
     * Display unresolved symbols or not.
     */
    public void displayUnresolved(boolean yes) {
        expander_.displayUnresolved(yes);
    }


    public void defineDataSet(String name, DataSet ds) {
        expander_.addDataSet(name, ds);
    }

    public void defineDataObject(String name, DataObject t) {
        expander_.addDataObject(name, t);
    }

    /**
     * Expand and display a database HTML template.
     * The SQL query must be present in the template.
     * @param out The PrintWriter where the HTML will be displayed
     * @param template The database HTML template to expand.
     */
    public void displayDbForm(PrintWriter out, String template)
            throws SQLException, IOException {

        String root = session_.getAppConfiguration().getRootDir();
        String fnm;
        if (root.endsWith(File.separator))
            fnm = new String(root + template);
        else
            fnm = new String(root + File.separator + template);

        fnm =
                fnm.replace('/', File.separatorChar).replace('\\', File.separatorChar);
        expander_.setInputFile(new File(fnm));
        try { expander_.expand(out); } catch(Exception ignore) { }
    }


    /**
     * Execute an update statement in the database, using an update template
     * and an HTTP request.<br>
     * Example:<br>
     * <ul>
     * <li>We need to update a customer's address in the database (example:
     * UPDATE Customer SET Address='12 Downing Street' WHERE LastName='Smith')
     * <li>An incoming HTTP request (from a submitted form) contains values
     * for the customer's address and last name (let's say "Add" and "Name").</li>
     * </ul>
     * In that case, the template string will be the following:<br>
     * UPDATE Customer SET Address='$Add$' WHERE LastName='$Name:  $'
     * @param req An HTTP request that contains parameter values
     * @param tmpl An SQL update template string.
     */
    public void updateDB(ESServletRequest req, String tmpl)
            throws IOException, SQLException, Exception {

        DynHtml d = new ESDynHtml(null, null);
        d.displayUnresolved(false);
        session_.updateDB(d.expandString(tmpl, req.getAllParams()));
    }

}

;

