/*
 * OpenEAS Experlog Application Server
 * Web application development and dynamic template scripting engine
 * Copyright (c) 2004 ExperLog SARL - http://www.experlog.com
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details. You should have
 * received a copy of the GNU General Public License along with this program;
 * if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.experlog.gencode;

import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import org.experlog.base.OpenClient;
import org.experlog.openeas.api.DataObject;

/**
 * @author gibello
 *
 */
public class ResourceBundleAccess implements DataObject {

  ESDynHtml page_ = null;
  OpenClient session_ = null;
  Locale currentLocale_ = null;
  Hashtable rbcache_ = new Hashtable();
  
  public ResourceBundleAccess(ESDynHtml page) {
    session_ = page.getSession();
    page_ = page;
    currentLocale_ = session_.getAppConfiguration().getLocale();
  }

  /**
   * Get string from ResourceBundle (for translation). The default ResourceBundleName is "MainBundle".
   * @param name The name of the message to retrieve (format : [ResourceBundleName.]Varname)
   * @see org.experlog.openeas.api.DataObject#get(java.lang.String)
   */
  public String get(String name) throws SQLException {
    if(name == null) return null;
    int pos = name.lastIndexOf('.');
    String bundlename = "MainBundle";
    if(pos >= 0) {
      if(pos > 0) bundlename = name.substring(0, pos);
      name = name.substring(pos+1);
    }
    
    ResourceBundle bundle = getBundle(bundlename);
    return (bundle == null ? null : page_.expandString(bundle.getString(name)));
  }

  private ResourceBundle getBundle(String name) {
    ResourceBundle bundle = null;
    Locale appl = session_.getAppConfiguration().getLocale();
    if(currentLocale_ != appl) {
      bundle = ResourceBundle.getBundle(name, appl);
      rbcache_.put(name, bundle);
      currentLocale_ = appl;
      return bundle;
    }
    
    bundle = (ResourceBundle)rbcache_.get(name);
    if(bundle == null) {
      bundle = ResourceBundle.getBundle(name, appl);
      rbcache_.put(name, bundle);
    }
    return bundle;
  }

}
